import java.io.*;


class MSIEParser  {
//
// MSIEParser.java  -- Parses Internet Explorer Shortcuts
//                     using StreamTokenizer in a state machine
//
// David Moisan, April 1st, 1999
// Version 1.1
//
// This code is freely distributable under the terms of the 
// GNU Public License (http://www.gnu.org/)
//
// Notes:
//
// Internet Shortcut files are in the form:
//
// [InternetShortcut]
//	URL=http://www.somewhere.com/~someone
//
// 
// Note the lack of quotes on the "http://..."
// 
// IMPORTANT:  Internet Explorer 5 has changed the bookmark format
// slightly.  The new format is as follows:
//
// [DEFAULT]
// BASEURL=http://www.w3.org/People/Raggett/tidy/
//
// [InternetShortcut]
// URL=http://www.w3.org/People/Raggett/tidy/
// Modified=E0C40E91D47BBE0151
//
//
// MSIEParser will process either the [DEFAULT] entry or the // [InternetExplorer] entry;  they appear to reference the same URL 
// --so far as I know.
// 
// MSIEParser ignores the [DEFAULT] entry 
//
// 
// This program was originally written for Java 1.0.2;  it runs
// under 1.1.6 but the StreamTokenizer object has been deprecated.
// I don't know when I will get the time to fix this...
//
//
// Labels for the state-machine parser
//

static final int START = 0;    // START: Initial state, nothing read
static final int HEADER = 1;   // HEADER: Header found
static final int URL = 2;      // URL: "URL" found 
static final int END = 10;     // END: URL found succesfully, end state
static final int ERROR = 1000; // ERROR: Elements not found or parser error
 

String MSIEGetURL(String filename) throws IOException {

//
// Open Internet Shortcut file and initialize;
//
   
	FileInputStream IEStream = new FileInputStream(filename);	
	StreamTokenizer IEP = new StreamTokenizer(IEStream);

//
// Initialize parser
//

	IEP.resetSyntax(); // All characters special
	IEP.eolIsSignificant(true); // return end of line (EOL)
	IEP.whitespaceChars(0x11, 0x20); // Whitespace is " "
	IEP.wordChars(0x21,0x7E); // All ASCII printable chars are words
	IEP.ordinaryChar((int)'=');	// Equal sign is ignored
	IEP.ordinaryChar((int)'\"'); // Same with quotes   
   
   int ParseState = START;
	String URLString = " ";
	int i;

//
// Parsing loop
//

	while((ParseState != ERROR) && (ParseState != END)) {
      IEP.nextToken();
	
		switch(IEP.ttype) {
			case IEP.TT_WORD: 
				
				switch (ParseState) {
				   case START: 
					   if (IEP.sval.indexOf("[InternetShortcut]") != -1) 
							ParseState = HEADER;
						break;

					case HEADER:
						if (IEP.sval.indexOf("URL")!= -1)
							ParseState = URL;
						break;
	
					case URL:
						URLString = IEP.sval; 
						// We've gotten the URL, we're done
						ParseState = END;
						break; 
											
					}
	
			case IEP.TT_EOL: {
				break;
				}
			
			case IEP.TT_EOF: {
				if (ParseState != END)	
					ParseState = ERROR;
				break;			
				}
						
         default:
            
            // Ignore all other cases for now
				break;
         }
		}
	if (ParseState == ERROR) {
		IEStream.close();
		IEStream = null;	
		throw new BadIEShortcut("URL not found in file: "+filename);
		}
	else if (ParseState != END) {
		IEStream.close();
		IEStream = null;			
		throw new BadIEShortcut
			("Parser error, ParseState= "+ParseState+" File:"+filename);
		}	
		
	IEStream.close();	
	IEStream = null;
	return URLString;
	}
}

//
// BadIEShortcut -- defines exception thrown in MSIEParser
//

class BadIEShortcut extends IOException {
	public BadIEShortcut() {
		super();
		}
	public BadIEShortcut(String s) {
		super(s);
		}
}